package org.elsys.pluginsample.editor.undoredo;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

/**
 * @author kiko
 * 
 */
public class ListEditor extends EditorPart {

	private List<String> fContent;

	private boolean fIsDirty;

	private ListViewer fViewer;

	protected AddAction fAddAction;

	protected DeleteAction fDeleteAction;

	private IFile fInputFile;

	/**
	 * 
	 */
	public ListEditor() {
	}

	public void doSave(IProgressMonitor monitor) {
		IWorkspaceRunnable runnable = createFileSaveRunnable();
		try {
			ResourcesPlugin.getWorkspace().run(runnable, monitor);
			fIsDirty = false;
			firePropertyChange(IEditorPart.PROP_DIRTY);
		} catch (CoreException e) {
			e.printStackTrace();
		}
	}

	private IWorkspaceRunnable createFileSaveRunnable() {
		IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
			public void run(IProgressMonitor monitor) throws CoreException {
				try {
					monitor.beginTask("Saving file...", 10);
					monitor.worked(2);
					if (monitor.isCanceled())
						throw new OperationCanceledException();
					ByteArrayOutputStream os = new ByteArrayOutputStream();
					ObjectOutputStream oos = new ObjectOutputStream(os);
					oos.writeObject(fContent);
					ByteArrayInputStream is = new ByteArrayInputStream(os
							.toByteArray());
					oos.close();
					os.close();
					fInputFile.setContents(is, IResource.FORCE,
							new SubProgressMonitor(monitor, 9));
				} catch (Exception e) {
					e.printStackTrace();
				} finally {
					monitor.done();
				}
			}
		};
		return runnable;
	}

	public void doSaveAs() {
	}

	public ListViewer getViewer() {
		return fViewer;
	}

	public List<String> getContent() {
		return fContent;
	}

	@SuppressWarnings("unchecked")
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		fInputFile = (IFile) input.getAdapter(IFile.class);
		if (fInputFile == null)
			throw new IllegalArgumentException(
					"The input must be adaptable to IFile");
		try {
			InputStream in = fInputFile.getContents();
			ObjectInputStream objectStream = new ObjectInputStream(in);
			fContent = (List<String>) objectStream.readObject();
		} catch (Exception e) {
			e.printStackTrace();
			fContent = new ArrayList<String>();
		}
		fIsDirty = false;
		setSite(site);
		setInput(input);
	}

	public boolean isDirty() {
		return fIsDirty;
	}

	public boolean isSaveAsAllowed() {
		return false;
	}

	public void setDirty() {
		fIsDirty = true;
		firePropertyChange(IEditorPart.PROP_DIRTY);
	}

	public void createPartControl(Composite parent) {
		Composite composite = new Group(parent, SWT.NONE);
		composite.setLayout(new GridLayout(1, false));
		composite.setLayoutData(new GridData(GridData.FILL));
		fillBody(composite);
		createActions();
		createContextMenu();
	}

	protected void createActions() {
		fAddAction = new AddAction(this);
		fDeleteAction = new DeleteAction(this);
	}

	private void createContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager m) {
				ListEditor.this.fillContextMenu(m);
			}
		});
		org.eclipse.swt.widgets.List list = fViewer.getList();
		Menu menu = menuMgr.createContextMenu(list);
		list.setMenu(menu);
		getSite().registerContextMenu(menuMgr, fViewer);
	}

	protected void fillContextMenu(IMenuManager menuMgr) {
		menuMgr.add(fAddAction);
		menuMgr.add(fDeleteAction);
		menuMgr.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	protected void fillBody(Composite parent) {
		ListViewer viewer = new ListViewer(parent, SWT.BORDER | SWT.SINGLE);
		GridData data = new GridData(GridData.FILL_BOTH);
		viewer.getList().setLayoutData(data);
		viewer.setContentProvider(new ListContentProvider());
		viewer.setLabelProvider(new LabelProvider());
		viewer.setInput(fContent);
		fViewer = viewer;
	}

	private static class ListContentProvider implements
			IStructuredContentProvider {
		@SuppressWarnings("unchecked")
		public Object[] getElements(Object inputElement) {
			if (inputElement instanceof List) {
				List list = (List) inputElement;
				return list.toArray(new Object[list.size()]);
			}
			throw new IllegalArgumentException("Unsupported input element");
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		}
	}

	public void setFocus() {
	}
}
